"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""
# from pytrip.ctx import *
# from pytrip.dos import *
# from pytrip.vdx import *
# import pytrip.dicomhelper as dh
# from pytrip.res.point import *

# dcm = dh.read_dicom_folder(
# "../../../DicomData/Til Jacob T anonymiseret/Anonymiseret/")
# c = CtxCube()
# c.read_dicom(dcm)
#
# v = VdxCube("",c)
# v.read_dicom(dcm)
