"""
    This file is part of PyTRiP.

    PyTRiP is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PyTRiP is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PyTRiP.  If not, see <http://www.gnu.org/licenses/>
"""

# do not check this file for PEP8 compatibility
# flake8: noqa

from pytrip.ctx import CtxCube
from pytrip.dos import DosCube
from pytrip.vdx import VdxCube
from pytrip.paths import DensityCube
from pytrip.rst import Rst
from pytrip.let import LETCube
from pytrip.get_dvh import GetDvh

__all__ = ['CtxCube', 'VdxCube', 'GetDvh', 'DosCube', 'DensityCube', 'LETCube', 'dicomhelper', 'res', 'guiutil', 'Rst']

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions
